Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const core = require('@sentry/core');
const debugBuild = require('../common/debug-build.js');
const utils = require('../common/utils.js');

/**
 * Takes a request event and extracts traceparent and DSC data
 * from the `sentry-trace` and `baggage` DSC headers.
 *
 * Sets propagation context as a side effect.
 */
function getTracePropagationData(event) {
  const sentryTrace = event.request.headers.get('sentry-trace') || '';
  const baggage = event.request.headers.get('baggage');

  return { sentryTrace, baggage };
}

/** Flush the event queue to ensure that events get sent to Sentry before the response is finished and the lambda ends */
async function flushIfServerless() {
  if (typeof process === 'undefined') {
    return;
  }

  const platformSupportsStreaming = !process.env.LAMBDA_TASK_ROOT && !process.env.VERCEL;

  if (!platformSupportsStreaming) {
    try {
      debugBuild.DEBUG_BUILD && core.logger.log('Flushing events...');
      await core.flush(2000);
      debugBuild.DEBUG_BUILD && core.logger.log('Done flushing events');
    } catch (e) {
      debugBuild.DEBUG_BUILD && core.logger.log('Error while flushing events:\n', e);
    }
  }
}

/**
 * Extracts a server-side sveltekit error, filters a couple of known errors we don't want to capture
 * and captures the error via `captureException`.
 *
 * @param e error
 *
 * @returns an objectified version of @param e
 */
function sendErrorToSentry(e, handlerFn) {
  // In case we have a primitive, wrap it in the equivalent wrapper class (string -> String, etc.) so that we can
  // store a seen flag on it.
  const objectifiedErr = core.objectify(e);

  // The error() helper is commonly used to throw errors in load functions: https://kit.svelte.dev/docs/modules#sveltejs-kit-error
  // If we detect a thrown error that is an instance of HttpError, we don't want to capture 4xx errors as they
  // could be noisy.
  // Also the `redirect(...)` helper is used to redirect users from one page to another. We don't want to capture thrown
  // `Redirect`s as they're not errors but expected behaviour
  if (
    utils.isRedirect(objectifiedErr) ||
    (utils.isHttpError(objectifiedErr) && objectifiedErr.status < 500 && objectifiedErr.status >= 400)
  ) {
    return objectifiedErr;
  }

  core.captureException(objectifiedErr, {
    mechanism: {
      type: 'sveltekit',
      handled: false,
      data: {
        function: handlerFn,
      },
    },
  });

  return objectifiedErr;
}

exports.flushIfServerless = flushIfServerless;
exports.getTracePropagationData = getTracePropagationData;
exports.sendErrorToSentry = sendErrorToSentry;
//# sourceMappingURL=utils.js.map
