Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const autoInstrument = require('./autoInstrument.js');
const detectAdapter = require('./detectAdapter.js');
const sourceMaps = require('./sourceMaps.js');

const DEFAULT_PLUGIN_OPTIONS = {
  autoUploadSourceMaps: true,
  autoInstrument: true,
  debug: false,
};

/**
 * Vite Plugins for the Sentry SvelteKit SDK, taking care of creating
 * Sentry releases and uploading source maps to Sentry.
 *
 * Sentry adds a few additional properties to your Vite config.
 * Make sure, it is registered before the SvelteKit plugin.
 */
async function sentrySvelteKit(options = {}) {
  const mergedOptions = {
    ...DEFAULT_PLUGIN_OPTIONS,
    ...options,
    adapter: options.adapter || (await detectAdapter.detectAdapter(options.debug)),
  };

  const sentryPlugins = [];

  if (mergedOptions.autoInstrument) {
    const pluginOptions = {
      load: true,
      serverLoad: true,
      ...(typeof mergedOptions.autoInstrument === 'object' ? mergedOptions.autoInstrument : {}),
    };

    sentryPlugins.push(
      autoInstrument.makeAutoInstrumentationPlugin({
        ...pluginOptions,
        debug: options.debug || false,
      }),
    );
  }

  const sentryVitePluginsOptions = generateVitePluginOptions(mergedOptions);

  if (sentryVitePluginsOptions) {
    const sentryVitePlugins = await sourceMaps.makeCustomSentryVitePlugins(sentryVitePluginsOptions);

    sentryPlugins.push(...sentryVitePlugins);
  }

  return sentryPlugins;
}

/**
 * This function creates the options for the custom Sentry Vite plugin.
 * The options are derived from the Sentry SvelteKit plugin options, where the `_unstable` options take precedence.
 *
 * only exported for testing
 */
function generateVitePluginOptions(
  svelteKitPluginOptions,
) {
  let sentryVitePluginsOptions = null;

  // Bundle Size Optimizations
  if (svelteKitPluginOptions.bundleSizeOptimizations) {
    sentryVitePluginsOptions = {
      bundleSizeOptimizations: {
        ...svelteKitPluginOptions.bundleSizeOptimizations,
      },
    };
  }

  // Source Maps
  if (svelteKitPluginOptions.autoUploadSourceMaps && process.env.NODE_ENV !== 'development') {
    const { unstable_sentryVitePluginOptions, ...sourceMapsUploadOptions } =
      svelteKitPluginOptions.sourceMapsUploadOptions || {};

    sentryVitePluginsOptions = {
      ...(sentryVitePluginsOptions ? sentryVitePluginsOptions : {}),

      ...sourceMapsUploadOptions,
      ...unstable_sentryVitePluginOptions,
      adapter: svelteKitPluginOptions.adapter,
      // override the plugin's debug flag with the one from the top-level options
      debug: svelteKitPluginOptions.debug,
    };

    if (sentryVitePluginsOptions.sourcemaps) {
      sentryVitePluginsOptions.sourcemaps = {
        ...sourceMapsUploadOptions?.sourcemaps,
        ...unstable_sentryVitePluginOptions?.sourcemaps,
      };
    }

    if (sentryVitePluginsOptions.release) {
      sentryVitePluginsOptions.release = {
        ...sourceMapsUploadOptions?.release,
        ...unstable_sentryVitePluginOptions?.release,
      };
    }
  }

  return sentryVitePluginsOptions;
}

exports.generateVitePluginOptions = generateVitePluginOptions;
exports.sentrySvelteKit = sentrySvelteKit;
//# sourceMappingURL=sentryVitePlugins.js.map
