Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const core = require('@sentry/core');
const node = require('@sentry/node');
const vitePlugin = require('@sentry/vite-plugin');
const child_process = require('child_process');
const fs = require('fs');
const MagicString = require('magic-string');
const path = require('path');
const utils = require('../common/utils.js');
const injectGlobalValues = require('./injectGlobalValues.js');
const svelteConfig = require('./svelteConfig.js');

/* eslint-disable max-lines */

// sorcery has no types, so these are some basic type definitions:

// storing this in the module scope because `makeCustomSentryVitePlugin` is called multiple times
// and we only want to generate a uuid once in case we have to fall back to it.
const releaseName = detectSentryRelease();

/**
 * Creates a new Vite plugin that uses the unplugin-based Sentry Vite plugin to create
 * releases and upload source maps to Sentry.
 *
 * Because the unplugin-based Sentry Vite plugin doesn't work ootb with SvelteKit,
 * we need to add some additional stuff to make source maps work:
 *
 * - the `config` hook needs to be added to generate source maps
 * - the `configResolved` hook tells us when to upload source maps.
 *   We only want to upload once at the end, given that SvelteKit makes multiple builds
 * - the `closeBundle` hook is used to flatten server source maps, which at the moment is necessary for SvelteKit.
 *   After the maps are flattened, they're uploaded to Sentry as in the original plugin.
 *   see: https://github.com/sveltejs/kit/discussions/9608
 *
 * @returns the custom Sentry Vite plugin
 */
async function makeCustomSentryVitePlugins(options) {
  const svelteConfig$1 = await svelteConfig.loadSvelteConfig();

  const usedAdapter = options?.adapter || 'other';
  const adapterOutputDir = await svelteConfig.getAdapterOutputDir(svelteConfig$1, usedAdapter);

  const defaultPluginOptions = {
    release: {
      name: releaseName,
    },
    _metaOptions: {
      telemetry: {
        metaFramework: 'sveltekit',
      },
    },
  };

  const { promise: filesToDeleteAfterUpload, resolve: resolveFilesToDeleteAfterUpload } =
    createFilesToDeleteAfterUploadPromise();

  const mergedOptions = {
    ...defaultPluginOptions,
    ...options,
    release: {
      ...defaultPluginOptions.release,
      ...options?.release,
    },
    sourcemaps: {
      ...options?.sourcemaps,
      filesToDeleteAfterUpload,
    },
  };

  const { debug } = mergedOptions;

  const sentryPlugins = await vitePlugin.sentryVitePlugin(mergedOptions);

  const sentryViteDebugIdUploadPlugin = sentryPlugins.find(
    plugin => plugin.name === 'sentry-vite-debug-id-upload-plugin',
  );

  const sentryViteFileDeletionPlugin = sentryPlugins.find(plugin => plugin.name === 'sentry-file-deletion-plugin');

  const sentryViteReleaseManagementPlugin = sentryPlugins.find(
    // sentry-debug-id-upload-plugin was the old (misleading) name of the plugin
    // sentry-release-management-plugin is the new name
    plugin => plugin.name === 'sentry-debug-id-upload-plugin' || plugin.name === 'sentry-release-management-plugin',
  );

  if (!sentryViteDebugIdUploadPlugin) {
    debug &&
      // eslint-disable-next-line no-console
      console.warn(
        'sentry-vite-debug-id-upload-plugin not found in sentryPlugins! Cannot modify plugin - returning default Sentry Vite plugins',
      );

    // resolving filesToDeleteAfterUpload here, because we return the original deletion plugin which awaits the promise
    resolveFilesToDeleteAfterUpload(undefined);

    return sentryPlugins;
  }

  if (!sentryViteFileDeletionPlugin) {
    debug &&
      // eslint-disable-next-line no-console
      console.warn(
        'sentry-file-deletion-plugin not found in sentryPlugins! Cannot modify plugin - returning default Sentry Vite plugins',
      );

    // resolving filesToDeleteAfterUpload here, because we return the original deletion plugin which awaits the promise
    resolveFilesToDeleteAfterUpload(undefined);

    return sentryPlugins;
  }

  if (!sentryViteReleaseManagementPlugin) {
    debug &&
      // eslint-disable-next-line no-console
      console.warn(
        'sentry-release-management-plugin not found in sentryPlugins! Cannot modify plugin - returning default Sentry Vite plugins',
      );

    // resolving filesToDeleteAfterUpload here, because we return the original deletion plugin which awaits the promise
    resolveFilesToDeleteAfterUpload(undefined);

    return sentryPlugins;
  }

  const unchangedSentryVitePlugins = sentryPlugins.filter(
    plugin =>
      ![
        'sentry-vite-debug-id-upload-plugin',
        'sentry-file-deletion-plugin',
        'sentry-release-management-plugin', // new name of release management plugin
        'sentry-debug-id-upload-plugin', // old name of release management plugin
      ].includes(plugin.name),
  );

  let isSSRBuild = true;

  const serverHooksFile = svelteConfig.getHooksFileName(svelteConfig$1, 'server');

  const globalSentryValues = {
    __sentry_sveltekit_output_dir: adapterOutputDir,
  };

  const sourceMapSettingsPlugin = {
    name: 'sentry-sveltekit-update-source-map-setting-plugin',
    apply: 'build', // only apply this plugin at build time
    config: async (config) => {
      const settingKey = 'build.sourcemap';

      const { updatedSourceMapSetting, previousSourceMapSetting } = getUpdatedSourceMapSetting(config);

      const userProvidedFilesToDeleteAfterUpload = await options?.sourcemaps?.filesToDeleteAfterUpload;

      if (previousSourceMapSetting === 'unset') {
        core.consoleSandbox(() => {
          //  eslint-disable-next-line no-console
          console.log(`[Sentry] Enabled source map generation in the build options with \`${settingKey}: "hidden"\`.`);
        });

        if (userProvidedFilesToDeleteAfterUpload) {
          resolveFilesToDeleteAfterUpload(userProvidedFilesToDeleteAfterUpload);
        } else {
          // Including all hidden (`.*`) directories by default so that folders like .vercel,
          // .netlify, etc are also cleaned up. Additionally, we include the adapter output
          // dir which could be a non-hidden directory, like `build` for the Node adapter.
          const defaultFileDeletionGlob = ['./.*/**/*.map', `./${adapterOutputDir}/**/*.map`];

          core.consoleSandbox(() => {
            // eslint-disable-next-line no-console
            console.warn(
              `[Sentry] Automatically setting \`sourceMapsUploadOptions.sourcemaps.filesToDeleteAfterUpload: [${defaultFileDeletionGlob
                .map(file => `"${file}"`)
                .join(', ')}]\` to delete generated source maps after they were uploaded to Sentry.`,
            );
          });

          // In case we enabled source maps and users didn't specify a glob patter to delete, we set a default pattern:
          resolveFilesToDeleteAfterUpload(defaultFileDeletionGlob);
        }

        return {
          ...config,
          build: { ...config.build, sourcemap: updatedSourceMapSetting },
        };
      }

      if (previousSourceMapSetting === 'disabled') {
        core.consoleSandbox(() => {
          //  eslint-disable-next-line no-console
          console.warn(
            `[Sentry] Parts of source map generation are currently disabled in your Vite configuration (\`${settingKey}: false\`). This setting is either a default setting or was explicitly set in your configuration. Sentry won't override this setting. Without source maps, code snippets on the Sentry Issues page will remain minified. To show unminified code, enable source maps in \`${settingKey}\` (e.g. by setting them to \`hidden\`).`,
          );
        });
      } else if (previousSourceMapSetting === 'enabled') {
        if (mergedOptions?.debug) {
          core.consoleSandbox(() => {
            // eslint-disable-next-line no-console
            console.log(
              `[Sentry] We discovered you enabled source map generation in your Vite configuration (\`${settingKey}\`). Sentry will keep this source map setting. This will un-minify the code snippet on the Sentry Issue page.`,
            );
          });
        }
      }

      resolveFilesToDeleteAfterUpload(userProvidedFilesToDeleteAfterUpload);

      return config;
    },
  };

  const customDebugIdUploadPlugin = {
    name: 'sentry-sveltekit-debug-id-upload-plugin',
    apply: 'build', // only apply this plugin at build time
    enforce: 'post', // this needs to be set to post, otherwise we don't pick up the output from the SvelteKit adapter
    resolveId: (id, _importer, _ref) => {
      if (id === injectGlobalValues.VIRTUAL_GLOBAL_VALUES_FILE) {
        return {
          id: injectGlobalValues.VIRTUAL_GLOBAL_VALUES_FILE,
          external: false,
          moduleSideEffects: true,
        };
      }
      return null;
    },

    load: id => {
      if (id === injectGlobalValues.VIRTUAL_GLOBAL_VALUES_FILE) {
        return {
          code: injectGlobalValues.getGlobalValueInjectionCode(globalSentryValues),
        };
      }
      return null;
    },

    configResolved: config => {
      // The SvelteKit plugins trigger additional builds within the main (SSR) build.
      // We just need a mechanism to upload source maps only once.
      // `config.build.ssr` is `true` for that first build and `false` in the other ones.
      // Hence we can use it as a switch to upload source maps only once in main build.
      if (!config.build.ssr) {
        isSSRBuild = false;
      }
    },

    transform: async (code, id) => {
      // eslint-disable-next-line @sentry-internal/sdk/no-regexp-constructor -- not end user input + escaped anyway
      const isServerHooksFile = new RegExp(`/${core.escapeStringForRegex(serverHooksFile)}(.(js|ts|mjs|mts))?`).test(id);

      if (isServerHooksFile) {
        const ms = new MagicString.default(code);
        ms.append(`\n; import "${injectGlobalValues.VIRTUAL_GLOBAL_VALUES_FILE}";\n`);
        return {
          code: ms.toString(),
          map: ms.generateMap({ hires: true }),
        };
      }

      return null;
    },

    // We need to start uploading source maps later than in the original plugin
    // because SvelteKit is invoking the adapter at closeBundle.
    // This means that we need to wait until the adapter is done before we start uploading.
    closeBundle: async () => {
      if (!isSSRBuild) {
        return;
      }

      const outDir = path.resolve(process.cwd(), adapterOutputDir);
      // eslint-disable-next-line no-console
      debug && console.log('[Source Maps Plugin] Looking up source maps in', outDir);

      const jsFiles = getFiles(outDir).filter(file => file.endsWith('.js'));
      // eslint-disable-next-line no-console
      debug && console.log('[Source Maps Plugin] Flattening source maps');

      // @ts-expect-error - we're using dynamic import here and TS complains about that. It works though.
      const sorcery = await import('sorcery');

      for (const file of jsFiles) {
        try {
          await (sorcery ).load(file).then(async chain => {
            if (!chain) {
              // We end up here, if we don't have a source map for the file.
              // This is fine, as we're not interested in files w/o source maps.
              return;
            }
            // This flattens the source map
            await chain.apply();
            // Write it back to the original file
            await chain.write();
          });
        } catch (e) {
          // Sometimes sorcery fails to flatten the source map. While this isn't ideal, it seems to be mostly
          // happening in Kit-internal files which is fine as they're not in-app.
          // This mostly happens when sorcery tries to resolve a source map while flattening that doesn't exist.
          const isKnownError = e instanceof Error && e.message.includes('ENOENT: no such file or directory, open');
          if (debug && !isKnownError) {
            // eslint-disable-next-line no-console
            console.error('[Source Maps Plugin] error while flattening', file, e);
          }
        }

        // We need to remove the query string from the source map files that our auto-instrument plugin added
        // to proxy the load functions during building.
        const mapFile = `${file}.map`;
        if (fs.existsSync(mapFile)) {
          const mapContent = (await fs.promises.readFile(mapFile, 'utf-8')).toString();
          const cleanedMapContent = mapContent.replace(
            // eslint-disable-next-line @sentry-internal/sdk/no-regexp-constructor -- no user input + escaped anyway
            new RegExp(core.escapeStringForRegex(utils.WRAPPED_MODULE_SUFFIX), 'gm'),
            '',
          );
          await fs.promises.writeFile(mapFile, cleanedMapContent);
        }
      }

      try {
        // So here, we're just calling the original plugin's `writeBundle` method to upload the source maps.
        // Our plugin hook expects output options to glob for source maps. We don't have this option in `closeBundle`.
        // So we just pass in the `outDir` we determined earlier.
        // Not pretty but my testing shows that it works.
        // @ts-expect-error - this hook exists on the plugin!
        await sentryViteDebugIdUploadPlugin.writeBundle({ dir: outDir });
      } catch (_) {
        // eslint-disable-next-line no-console
        console.warn('[Source Maps Plugin] Failed to upload source maps!');
        // eslint-disable-next-line no-console
        console.log(
          '[Source Maps Plugin] Please make sure you specified a valid Sentry auth token, as well as your org and project slugs.',
        );
        // eslint-disable-next-line no-console
        console.log(
          '[Source Maps Plugin] Further information: https://github.com/getsentry/sentry-javascript/blob/develop/packages/sveltekit/README.md#uploading-source-maps',
        );
      }
    },
  };

  // The file deletion plugin is originally called in `writeBundle`.
  // We need to call it in `closeBundle` though, because we also postpone
  // the upload step to `closeBundle`
  const customFileDeletionPlugin = {
    name: 'sentry-sveltekit-file-deletion-plugin',
    apply: 'build', // only apply this plugin at build time
    enforce: 'post',
    closeBundle: async () => {
      if (!isSSRBuild) {
        return;
      }

      const writeBundleFn = sentryViteFileDeletionPlugin?.writeBundle;
      if (typeof writeBundleFn === 'function') {
        // This is fine though, because the original method doesn't consume any arguments in its `writeBundle` callback.
        const outDir = path.resolve(process.cwd(), adapterOutputDir);
        try {
          // @ts-expect-error - the writeBundle hook expects two args we can't pass in here (they're only available in `writeBundle`)
          await writeBundleFn({ dir: outDir });
        } catch (e) {
          // eslint-disable-next-line no-console
          console.warn('Failed to delete source maps:', e);
        }
      }
    },
  };

  const customReleaseManagementPlugin = {
    name: 'sentry-sveltekit-release-management-plugin',
    apply: 'build', // only apply this plugin at build time
    enforce: 'post',
    closeBundle: async () => {
      try {
        // @ts-expect-error - this hook exists on the plugin!
        await sentryViteReleaseManagementPlugin.writeBundle();
      } catch (e) {
        // eslint-disable-next-line no-console
        console.warn('[Source Maps Plugin] Failed to upload release data:', e);
      }
    },
  };

  return [
    ...unchangedSentryVitePlugins,
    sourceMapSettingsPlugin,
    customReleaseManagementPlugin,
    customDebugIdUploadPlugin,
    customFileDeletionPlugin,
  ];
}

/**
 * Whether the user enabled (true, 'hidden', 'inline') or disabled (false) source maps
 */

/** There are 3 ways to set up source map generation (https://github.com/getsentry/sentry-javascript/issues/13993)
 *
 *     1. User explicitly disabled source maps
 *       - keep this setting (emit a warning that errors won't be unminified in Sentry)
 *       - We won't upload anything
 *
 *     2. Users enabled source map generation (true, 'hidden', 'inline').
 *       - keep this setting (don't do anything - like deletion - besides uploading)
 *
 *     3. Users didn't set source maps generation
 *       - we enable 'hidden' source maps generation
 *       - configure `filesToDeleteAfterUpload` to delete all .map files (we emit a log about this)
 *
 * --> only exported for testing
 */
function getUpdatedSourceMapSetting(viteConfig

) {
  viteConfig.build = viteConfig.build || {};

  const originalSourcemapSetting = viteConfig.build.sourcemap;

  if (originalSourcemapSetting === false) {
    return {
      previousSourceMapSetting: 'disabled',
      updatedSourceMapSetting: originalSourcemapSetting,
    };
  }

  if (originalSourcemapSetting && ['hidden', 'inline', true].includes(originalSourcemapSetting)) {
    return { previousSourceMapSetting: 'enabled', updatedSourceMapSetting: originalSourcemapSetting };
  }

  return {
    previousSourceMapSetting: 'unset',
    updatedSourceMapSetting: 'hidden',
  };
}

function getFiles(dir) {
  if (!fs.existsSync(dir)) {
    return [];
  }
  const dirents = fs.readdirSync(dir, { withFileTypes: true });
  // eslint-disable-next-line @typescript-eslint/ban-ts-comment
  // @ts-expect-error
  const files = dirents.map(dirent => {
    const resFileOrDir = path.resolve(dir, dirent.name);
    return dirent.isDirectory() ? getFiles(resFileOrDir) : resFileOrDir;
  });

  return Array.prototype.concat(...files);
}

function detectSentryRelease() {
  let releaseFallback;
  try {
    releaseFallback = child_process
      .execSync('git rev-parse HEAD', { stdio: ['ignore', 'pipe', 'ignore'] })
      .toString()
      .trim();
  } catch (_) {
    // the command can throw for various reasons. Most importantly:
    // - git is not installed
    // - there is no git repo or no commit yet
    // regardless of the case we just fall back to assigning a random uuid.
    releaseFallback = core.uuid4();
  }
  const release = node.getSentryRelease() || releaseFallback;

  return release;
}

/**
 * Creates a deferred promise that can be resolved/rejected by calling the
 * `resolve` or `reject` function.
 * Inspired by: https://stackoverflow.com/a/69027809
 */
function createFilesToDeleteAfterUploadPromise()

 {
  let resolve;
  let reject;
  const promise = new Promise((res, rej) => {
    resolve = res;
    reject = rej;
  });
  return { resolve, reject, promise };
}

exports.getUpdatedSourceMapSetting = getUpdatedSourceMapSetting;
exports.makeCustomSentryVitePlugins = makeCustomSentryVitePlugins;
//# sourceMappingURL=sourceMaps.js.map
