import { captureException, consoleSandbox } from '@sentry/core';
import { flushIfServerless } from './utils.js';

// The SvelteKit default error handler just logs the error's stack trace to the console
// see: https://github.com/sveltejs/kit/blob/369e7d6851f543a40c947e033bfc4a9506fdc0a8/packages/kit/src/runtime/server/index.js#L43
function defaultErrorHandler({ error }) {
  // @ts-expect-error this conforms to the default implementation (including this ts-expect-error)
  // eslint-disable-next-line no-console
  consoleSandbox(() => console.error(error?.stack));
}

/**
 * Wrapper for the SvelteKit error handler that sends the error to Sentry.
 *
 * @param handleError The original SvelteKit error handler.
 */
function handleErrorWithSentry(handleError = defaultErrorHandler) {
  return async (input) => {
    if (isNotFoundError(input)) {
      // We're extra cautious with SafeHandleServerErrorInput - this type is not compatible with HandleServerErrorInput
      // @ts-expect-error - we're still passing the same object, just with a different (backwards-compatible) type
      return handleError(input);
    }

    captureException(input.error, {
      mechanism: {
        type: 'sveltekit',
        handled: false,
      },
    });

    await flushIfServerless();

    // We're extra cautious with SafeHandleServerErrorInput - this type is not compatible with HandleServerErrorInput
    // @ts-expect-error - we're still passing the same object, just with a different (backwards-compatible) type
    return handleError(input);
  };
}

/**
 * When a page request fails because the page is not found, SvelteKit throws a "Not found" error.
 */
function isNotFoundError(input) {
  const { error, event, status } = input;

  // SvelteKit 2.0 offers a reliable way to check for a Not Found error:
  if (status === 404) {
    return true;
  }

  // SvelteKit 1.x doesn't offer a reliable way to check for a Not Found error.
  // So we check the route id (shouldn't exist) and the raw stack trace
  // We can delete all of this below whenever we drop Kit 1.x support
  const hasNoRouteId = !event.route?.id;

  const rawStack =
    (error != null &&
      typeof error === 'object' &&
      'stack' in error &&
      typeof error.stack === 'string' &&
      error.stack) ||
    '';

  return hasNoRouteId && rawStack.startsWith('Error: Not found:');
}

export { handleErrorWithSentry };
//# sourceMappingURL=handleError.js.map
